<?php

namespace Drupal\edlp_productions\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Url;
use Drupal\Core\Language\LanguageInterface;
use Drupal\menu_link_content\Entity\MenuLinkContent;

// use Symfony\Component\HttpFoundation\JsonResponse;
use Drupal\Core\Cache\CacheableJsonResponse;
use Drupal\Core\Cache\CacheableMetadata;
use Drupal\core\render\RenderContext;

class ProductionsController extends ControllerBase {

  private function query() {
    // TODO: get the production menu
    // @see https://api.drupal.org/api/drupal/core%21modules%21menu_ui%21menu_ui.module/8.6.x
    // @see https://drupal.stackexchange.com/questions/200893/get-menu-link-page-settings-from-view-configuration
    $menu_name = 'productions';

    $query = \Drupal::entityQuery('menu_link_content')
			->accessCheck(TRUE)
            ->condition('menu_name', $menu_name)
            ->condition('enabled', 1)
            ->sort('weight', 'ASC');

    $result = $query->execute();
    // dpm($result);

    $nids = [];
    foreach ($result as $id) {
      $menu_link = MenuLinkContent::load($id);
      // dpm($menu_link->getParentId());
      if($menu_link->getParentId() !== "") continue;

      // dpm($menu_link->getTitle());
      $path = $menu_link->getUrlObject()->getInternalPath();
      preg_match('/^(node)\/(\d+)$/', $path, $m);
      // dpm($m);
      if($m[1] !== "node") continue;

      $nids[] = ($m[2]);
    }

    $this->nodes = \Drupal::entityTypeManager()->getStorage('node')->loadMultiple($nids);

  }


  private function toRenderable(){
    $this->query();
    // dpm($this->future_nodes);

    // TODO: add localtasks to each nodes : https://api.drupal.org/api/drupal/core%21includes%21menu.inc/function/menu_primary_local_tasks/8.2.x

    return array(
      "#theme"=>'edlp_productions',
      "#nodes_entities"=>$this->nodes,
    );

  }

  /**
   * Display productions as a page.
   *
   * @return renderable array
   */
  public function productions() {
    // $this.getProductionBlock();
    return $this->toRenderable();
  }


  private function getProductionBlock(){
    $block = \Drupal\block\Entity\Block::load('productions');

    return $block;
  }

  /**
   * Get agenda data as json through ajax.
   *
   * @return json
   */
  public function productionsjson() {

    $renderable = $this->toRenderable();
    // $rendered = render($renderable);
    // We can't render directly the entity as it throw an exception with cachable data
    // see http://blog.dcycle.com/blog/2018-01-24/caching-drupal-8-rest-resource/#the-dreaded-leaked-metadata-error
    $rendered = \Drupal::service('renderer')->executeInRenderContext(new RenderContext(), function () use ($renderable) {
      return render($renderable);
    });

    // add menu production block
    // not used anymore as production block is always present (but not visible)
    // $block = $this->getProductionBlock();
    // $block_render = \Drupal::entityTypeManager()
    //   ->getViewBuilder('block')
    //   ->view($block);

    $data = [
      'rendered'=> $rendered,
      'title'=> 'Productions',
      // 'block' => array(
      //   'rendered'=> \Drupal::service('renderer')->renderRoot($block_render),
      //   'region'=> str_replace('_', '-', $block->getRegion()),
      //   'id'=> preg_replace('/^[^:]+:(.+)$/', 'block-$1', $block->getPluginId())
      // )
    ];

    // translations links
    $route_name = 'edlp_productions.productions';
    $links = \Drupal::languageManager()->getLanguageSwitchLinks(LanguageInterface::TYPE_URL, Url::fromRoute($route_name));
    if (isset($links->links)) {
      $translations_build = [
        '#theme' => 'links__language_block',
        '#links' => $links->links,
        '#attributes' => ['class' => ["language-switcher-{$links->method_id}",],],
        '#set_active_class' => TRUE,
      ];
      $translations_rendered = \Drupal::service('renderer')->executeInRenderContext(new RenderContext(), function () use ($translations_build) {return render($translations_build);});

      $data ['translations_links'] = $translations_rendered;
    }

    $data['#cache'] = [
      'max-age' => \Drupal\Core\Cache\Cache::PERMANENT,
      'tags' => ['edlp-productions-cache']
    ];

    // $response = new JsonResponse();
    // $response->setData($data);
    $response = new CacheableJsonResponse($data);
    $response->addCacheableDependency(CacheableMetadata::createFromRenderArray($data));
    $response->addCacheableDependency(CacheableMetadata::createFromRenderArray($renderable));

    return $response;
    // return array(
    //   '#markup'=>'hello'
    // );
  }

}
